@@ -1,6 +1,4 @@
-# -*- coding: utf-8 -*-
-#
-# Copyright 2010-2019 The pygit2 contributors
+# Copyright 2010-2020 The pygit2 contributors
 #
 # This file is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2,
@@ -25,21 +23,20 @@
 # the Free Software Foundation, 51 Franklin Street, Fifth Floor,
 # Boston, MA 02110-1301, USA.
 
-# Import from the future
-from __future__ import absolute_import, unicode_literals
+from cached_property import cached_property
 
 # Import from pygit2
 from .errors import check_error
 from .ffi import ffi, C
-from .utils import to_bytes, is_string
+from .utils import to_bytes
 
 
 def assert_string(v, desc):
-    if not is_string(v):
+    if not isinstance(v, str):
         raise TypeError("%s must be a string" % desc)
 
 
-class ConfigIterator(object):
+class ConfigIterator:
 
     def __init__(self, config, ptr):
         self._iter = ptr
@@ -56,14 +53,13 @@ class ConfigIterator(object):
         err = C.git_config_next(centry, self._iter)
         check_error(err)
 
-        return ConfigEntry._from_c(centry[0], True)
+        return ConfigEntry._from_c(centry[0], self)
 
     def next(self):
         return self.__next__()
 
     def __next__(self):
-        entry = self._next_entry()
-        return entry
+        return self._next_entry()
 
 
 class ConfigMultivarIterator(ConfigIterator):
@@ -72,8 +68,9 @@ class ConfigMultivarIterator(ConfigIterator):
         return entry.value
 
 
-class Config(object):
-    """Git configuration management"""
+class Config:
+    """Git configuration management.
+    """
 
     def __init__(self, path=None):
         cconfig = ffi.new('git_config **')
@@ -84,7 +81,7 @@ class Config(object):
             assert_string(path, "path")
             err = C.git_config_open_ondisk(cconfig, to_bytes(path))
 
-        check_error(err, True)
+        check_error(err, io=True)
         self._config = cconfig[0]
 
     @classmethod
@@ -96,7 +93,10 @@ class Config(object):
         return config
 
     def __del__(self):
-        C.git_config_free(self._config)
+        try:
+            C.git_config_free(self._config)
+        except AttributeError:
+            pass
 
     def _get(self, key):
         assert_string(key, "key")
@@ -195,7 +195,7 @@ class Config(object):
 
         entry = self._get_entry(key)
         res = ffi.new('int *')
-        err = C.git_config_parse_bool(res, entry.raw_value)
+        err = C.git_config_parse_bool(res, entry.c_value)
         check_error(err)
 
         return res[0] != 0
@@ -210,7 +210,7 @@ class Config(object):
 
         entry = self._get_entry(key)
         res = ffi.new('int64_t *')
-        err = C.git_config_parse_int64(res, entry.raw_value)
+        err = C.git_config_parse_int64(res, entry.c_value)
         check_error(err)
 
         return res[0]
@@ -262,7 +262,7 @@ class Config(object):
     def _from_found_config(fn):
         buf = ffi.new('git_buf *', (ffi.NULL, 0))
         err = fn(buf)
-        check_error(err, True)
+        check_error(err, io=True)
         cpath = ffi.string(buf.ptr).decode('utf-8')
         C.git_buf_dispose(buf)
 
@@ -286,51 +286,63 @@ class Config(object):
         """
         return Config._from_found_config(C.git_config_find_xdg)
 
-class ConfigEntry(object):
-    """An entry in a configuation object
+
+class ConfigEntry:
+    """An entry in a configuation object.
     """
 
     @classmethod
-    def _from_c(cls, ptr, from_iterator=False):
-        """Builds the entry from a ``git_config_entry`` pointer. ``from_iterator``
-        should be ``True`` when the entry was created during ``git_config_iterator``
-        actions
+    def _from_c(cls, ptr, iterator=None):
+        """Builds the entry from a ``git_config_entry`` pointer.
+
+        ``iterator`` must be a ``ConfigIterator`` instance if the entry was
+        created during ``git_config_iterator`` actions.
         """
         entry = cls.__new__(cls)
         entry._entry = ptr
-        entry.from_iterator = from_iterator
+        entry.iterator = iterator
+
+        # It should be enough to keep a reference to iterator, so we only call
+        # git_config_iterator_free when we've deleted all ConfigEntry objects.
+        # But it's not, to reproduce the error comment the lines below and run
+        # the script in https://github.com/libgit2/pygit2/issues/970
+        # So instead we load the Python object immmediately. Ideally we should
+        # investigate libgit2 source code.
+        if iterator is not None:
+            entry.raw_name = entry.raw_name
+            entry.raw_value = entry.raw_value
+            entry.level = entry.level
+
         return entry
 
     def __del__(self):
-        if not self.from_iterator:
+        if self.iterator is None:
             C.git_config_entry_free(self._entry)
 
     @property
-    def value(self):
-        """The entry's value as a string
-        """
-        return self.decode_as_string(self._entry.value)
+    def c_value(self):
+        """The raw ``cData`` entry value."""
+        return self._entry.value
 
-    @property
+    @cached_property
+    def raw_name(self):
+        return ffi.string(self._entry.name)
+
+    @cached_property
+    def raw_value(self):
+        return ffi.string(self.c_value)
+
+    @cached_property
     def level(self):
-        """The entry's ``git_config_level_t`` value
-        """
+        """The entry's ``git_config_level_t`` value."""
         return self._entry.level
 
     @property
     def name(self):
-        """The entry's name
-        """
-        return self.decode_as_string(self._entry.name)
+        """The entry's name."""
+        return self.raw_name.decode('utf-8')
 
     @property
-    def raw_value(self):
-        """The raw ``cData`` entry value
-        """
-        return self._entry.value
-
-    @staticmethod
-    def decode_as_string(value):
-        """Returns ``value`` as a decoded ``utf-8`` string.
-        """
-        return ffi.string(value).decode('utf-8')
+    def value(self):
+        """The entry's value as a string."""
+        return self.raw_value.decode('utf-8')
