@@ -1,6 +1,4 @@
-# -*- coding: utf-8 -*-
-#
-# Copyright 2010-2017 The pygit2 contributors
+# Copyright 2010-2020 The pygit2 contributors
 #
 # This file is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2,
@@ -25,20 +23,17 @@
 # the Free Software Foundation, 51 Franklin Street, Fifth Floor,
 # Boston, MA 02110-1301, USA.
 
-# Import from the future
-from __future__ import absolute_import, unicode_literals
-
 import weakref
 
 # Import from pygit2
-from _pygit2 import Oid, Tree, Diff
+from ._pygit2 import Oid, Tree, Diff
 from .errors import check_error
 from .ffi import ffi, C
-from .utils import is_string, to_bytes, to_str
+from .utils import to_bytes, to_str
 from .utils import GenericIterator, StrArray
 
 
-class Index(object):
+class Index:
 
     def __init__(self, path=None):
         """Create a new Index
@@ -83,12 +78,15 @@ class Index(object):
 
     def __getitem__(self, key):
         centry = ffi.NULL
-        if is_string(key):
+        if isinstance(key, str) or hasattr(key, '__fspath__'):
             centry = C.git_index_get_bypath(self._index, to_bytes(key), 0)
-        elif not key >= 0:
-            raise ValueError(key)
+        elif isinstance(key, int):
+            if key >= 0:
+                centry = C.git_index_get_byindex(self._index, key)
+            else:
+                raise ValueError(key)
         else:
-            centry = C.git_index_get_byindex(self._index, key)
+            raise TypeError('Expected str or int, got %s' % type(key))
 
         if centry == ffi.NULL:
             raise KeyError(key)
@@ -105,17 +103,17 @@ class Index(object):
         Parameters:
 
         force
-            If True (the default) allways reload. If False, only if the file
+            If True (the default) always reload. If False, only if the file
             has changed.
         """
 
         err = C.git_index_read(self._index, force)
-        check_error(err, True)
+        check_error(err, io=True)
 
     def write(self):
         """Write the contents of the Index to disk."""
         err = C.git_index_write(self._index)
-        check_error(err, True)
+        check_error(err, io=True)
 
     def clear(self):
         err = C.git_index_clear(self._index)
@@ -129,7 +127,7 @@ class Index(object):
         inserted into the Index.
         """
         repo = self._repo
-        if is_string(tree):
+        if isinstance(tree, str):
             tree = repo[tree]
 
         if isinstance(tree, Oid):
@@ -172,7 +170,14 @@ class Index(object):
         """Remove an entry from the Index.
         """
         err = C.git_index_remove(self._index, to_bytes(path), level)
-        check_error(err, True)
+        check_error(err, io=True)
+
+    def remove_all(self, pathspecs):
+        """Remove all index entries matching pathspecs.
+        """
+        with StrArray(pathspecs) as arr:
+            err = C.git_index_remove_all(self._index, arr, ffi.NULL, ffi.NULL)
+            check_error(err, io=True)
 
     def add_all(self, pathspecs=[]):
         """Add or update index entries matching files in the working directory.
@@ -182,7 +187,7 @@ class Index(object):
         """
         with StrArray(pathspecs) as arr:
             err = C.git_index_add_all(self._index, arr, 0, ffi.NULL, ffi.NULL)
-            check_error(err, True)
+            check_error(err, io=True)
 
     def add(self, path_or_entry):
         """Add or update an entry in the Index.
@@ -195,17 +200,17 @@ class Index(object):
         Index without checking for the existence of the path or id.
         """
 
-        if is_string(path_or_entry):
-            path = path_or_entry
-            err = C.git_index_add_bypath(self._index, to_bytes(path))
-        elif isinstance(path_or_entry, IndexEntry):
+        if isinstance(path_or_entry, IndexEntry):
             entry = path_or_entry
             centry, str_ref = entry._to_c()
             err = C.git_index_add(self._index, centry)
+        elif isinstance(path_or_entry, str) or hasattr(path_or_entry, '__fspath__'):
+            path = path_or_entry
+            err = C.git_index_add_bypath(self._index, to_bytes(path))
         else:
             raise AttributeError('argument must be string or IndexEntry')
 
-        check_error(err, True)
+        check_error(err, io=True)
 
     def diff_to_workdir(self, flags=0, context_lines=3, interhunk_lines=0):
         """
@@ -304,13 +309,13 @@ class Index(object):
         that represents the conflicts in the index.
 
         This object presents a mapping interface with the paths as keys. You
-        can use the ``del`` operator to remove a conflict form the Index.
+        can use the ``del`` operator to remove a conflict from the Index.
 
         Each conflict is made up of three elements. Access or iteration
         of the conflicts returns a three-tuple of
         :py:class:`~pygit2.IndexEntry`. The first is the common
-        ancestor, the second is the "ours" side of the conflict and the
-        thirs is the "theirs" side.
+        ancestor, the second is the "ours" side of the conflict, and the
+        third is the "theirs" side.
 
         These elements may be None depending on which sides exist for
         the particular conflict.
@@ -327,7 +332,7 @@ class Index(object):
         return self._conflicts()
 
 
-class IndexEntry(object):
+class IndexEntry:
     __slots__ = ['id', 'path', 'mode']
 
     def __init__(self, path, object_id, mode):
@@ -376,7 +381,7 @@ class IndexEntry(object):
         return entry
 
 
-class ConflictCollection(object):
+class ConflictCollection:
 
     def __init__(self, index):
         self._index = index
@@ -404,7 +409,7 @@ class ConflictCollection(object):
         return ConflictIterator(self._index)
 
 
-class ConflictIterator(object):
+class ConflictIterator:
 
     def __init__(self, index):
         citer = ffi.new('git_index_conflict_iterator **')
