@@ -1,6 +1,4 @@
-# -*- coding: utf-8 -*-
-#
-# Copyright 2010-2019 The pygit2 contributors
+# Copyright 2010-2020 The pygit2 contributors
 #
 # This file is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2,
@@ -25,17 +23,16 @@
 # the Free Software Foundation, 51 Franklin Street, Fifth Floor,
 # Boston, MA 02110-1301, USA.
 
-# Import from the future
-from __future__ import absolute_import
-
 # Import from pygit2
 from .errors import check_error
 from .ffi import ffi, C
 from .utils import to_bytes
 
 
-class Refspec(object):
-    """The constructor is for internal use only"""
+class Refspec:
+    """The constructor is for internal use only.
+    """
+
     def __init__(self, owner, ptr):
         self._owner = owner
         self._refspec = ptr
